<?php
namespace App\PulpoThirdParty;

use ArrayAccess;
use Exception;
use Iterator;

class PulpoRestClientException extends Exception {}

class PulpoRestClient implements Iterator, ArrayAccess {

	public $options;
	public $handle;

	public $response;
	public $headers;
	public $info;
	public $error;
	public $response_status_lines;
	
	public $decoded_response;

	public function __construct( array $options = []) {
		$default_options = [
			'headers' => [], 
			'parameters' => [], 
			'curl_options' => [], 
			'build_indexed_queries' => false, 
			'user_agent' => 'PHP RestClient/0.1.8',
			'base_url' => null, 
			'format' => null, 
			'format_regex' => '/(\w+)\/(\w+)(;[.+])?/',
			'decoders' => [
				'json' => 'json_decode',
				'php' => 'unserialize'
			],
			'username' => null,
			'password' => null
		];

		$this->options = array_merge($default_options, $options);
		if (array_key_exists('decoders', $options)) {
			$this->options['decoders'] = array_merge(
				$default_options['decoders'], $options['decoders']);
		}
	}
	
	public function set_option( $key, $value) {
		$this->options[$key] = $value;
	}

	public function register_decoder( $format, callable $method) {
		$this->options['decoders'][$format] = $method;
	}
	
	public function rewind() {
		$this->decode_response();
		reset($this->decoded_response);
	}
	
	public function current() {
		return current($this->decoded_response);
	}
	
	public function key() {
		return key($this->decoded_response);
	}
	
	public function next() {
		next($this->decoded_response);
	}
	
	public function valid() {
		return is_array($this->decoded_response)
			&& ( key($this->decoded_response) !== null );
	}
	
	public function offsetExists( $key) {
		$this->decode_response();
		return is_array($this->decoded_response)?
			isset($this->decoded_response[$key]) : isset($this->decoded_response->{$key});
	}
	
	public function offsetGet( $key) {
		$this->decode_response();
		if (!$this->offsetExists($key)) {
			return null;
		}
		
		return is_array($this->decoded_response)?
			$this->decoded_response[$key] : $this->decoded_response->{$key};
	}

	public function offsetSet( $key, $value) {
		throw new PulpoRestClientException('Decoded response data is immutable.');
	}
	
	public function offsetUnset( $key) {
		throw new PulpoRestClientException('Decoded response data is immutable.');
	}

	public function get( $url, $parameters = [], array $headers = []) {
		return $this->execute($url, 'GET', $parameters, $headers);
	}
	
	public function post( $url, $parameters = [], array $headers = []) {
		return $this->execute($url, 'POST', $parameters, $headers);
	}

	public function put( $url, $parameters = [], array $headers = []) {
		return $this->execute($url, 'PUT', $parameters, $headers);
	}
	
	public function patch( $url, $parameters = [], array $headers = []) {
		return $this->execute($url, 'PATCH', $parameters, $headers);
	}
	
	public function delete( $url, $parameters = [], array $headers = []) {
		return $this->execute($url, 'DELETE', $parameters, $headers);
	}

	public function head( $url, $parameters = [], array $headers = []) {
		return $this->execute($url, 'HEAD', $parameters, $headers);
	}

	public function execute( $url, $method = 'GET', $parameters = [], array $headers = []) {
		$client = clone $this;
		$client->url = $url;
		$client->handle = curl_init();
		$curlopt = [
			CURLOPT_HEADER => true,
			CURLOPT_RETURNTRANSFER => true, 
			CURLOPT_USERAGENT => $client->options['user_agent']
		];

		if ($client->options['username'] && $client->options['password']) {
			$curlopt[CURLOPT_USERPWD] = sprintf('%s:%s', 
			$client->options['username'], $client->options['password']);
		}

		if ($client->options['format'] && 'json' === $client->options['format']) {
			$headers['content-type'] = 'application/json';
		}

		if (count($client->options['headers']) || count($headers)) {
			$curlopt[CURLOPT_HTTPHEADER] = [];
			$headers = array_merge($client->options['headers'], $headers);
			foreach ($headers as $key => $values) {
				foreach (is_array($values)? $values : [$values] as $value) {
					$curlopt[CURLOPT_HTTPHEADER][] = sprintf('%s:%s', $key, $value);
				}
			}
		}
		
		//if($client->options['format'])
		//    $client->url .= '.'.$client->options['format'];
		
		// Allow passing parameters as a pre-encoded string (or something that
		// allows casting to a string). Parameters passed as strings will not be
		// merged with parameters specified in the default options.
		if (is_array($parameters)) {
			$parameters = array_merge($client->options['parameters'], $parameters);
			$parameters_string = http_build_query($parameters);

			// http_build_query automatically adds an array index to repeated
			// parameters which is not desirable on most systems. This hack
			// reverts "key[0]=foo&key[1]=bar" to "key[]=foo&key[]=bar"
			if (!$client->options['build_indexed_queries']) {
				$parameters_string = preg_replace(
					'/%5B[0-9]+%5D=/simU', '%5B%5D=', $parameters_string);
			}
		} else {
			$parameters_string = (string) $parameters;
		}
		
		if (strtoupper($method) === 'POST' || strtoupper($method) === 'PUT') {
			if (strtoupper($method) === 'POST') {
				$curlopt[CURLOPT_POST] = true;
			} else {
				$curlopt[CURLOPT_CUSTOMREQUEST] = 'PUT';
			}

			if ($client->options['format'] && 'json' === $client->options['format']) {
				$curlopt[CURLOPT_POSTFIELDS] = json_encode(array_merge($client->options['parameters'], isset($parameters)? $parameters : []));
			} else {
				$curlopt[CURLOPT_POSTFIELDS] = $parameters_string;
			}
		} elseif (strtoupper($method) !== 'GET') {
			$curlopt[CURLOPT_CUSTOMREQUEST] = strtoupper($method);
			$curlopt[CURLOPT_POSTFIELDS] = $parameters_string;
		} elseif ($parameters_string) {
			$client->url .= strpos($client->url, '?')? '&' : '?';
			$client->url .= $parameters_string;
		}

		if ($client->options['base_url']) {
			if ('/' !== $client->url[0] && '/' !== substr($client->options['base_url'], -1)) {
				$client->url = '/' . $client->url;
			}
			$client->url = $client->options['base_url'] . $client->url;
		}
		$curlopt[CURLOPT_URL] = $client->url;
		$curlopt[CURLOPT_SSL_VERIFYPEER] = 0;
		$curlopt[CURLOPT_SSL_VERIFYHOST] = 0;

		if ($client->options['curl_options']) {
			// array_merge would reset our numeric keys.
			foreach ($client->options['curl_options'] as $key => $value) {
				$curlopt[$key] = $value;
			}
		}
		curl_setopt_array($client->handle, $curlopt);
		
		$client->parse_response(curl_exec($client->handle));
		$client->info = (object) curl_getinfo($client->handle);
		$client->error = curl_error($client->handle);

		curl_close($client->handle);
		return $client;
	}
	
	public function parse_response( $response) {
		if (!$response) {
			return;
		}

		$headers = [];
		$this->response_status_lines = [];
		$line = strtok($response, "\n");
		do {
			if (strlen(trim($line)) == 0) {
				// Since we tokenize on \n, use the remaining \r to detect empty lines.
				if (count($headers) > 0) {
break; // Must be the newline after headers, move on to response body
				}
			} elseif (strpos($line, 'HTTP') === 0) {
				// One or more HTTP status lines
				$this->response_status_lines[] = trim($line);
			} else { 
				// Has to be a header
				list($key, $value) = explode(':', $line, 2);
				$key = strtolower(trim(str_replace('-', '_', $key)));
				$value = trim($value);
				
				if (empty($headers[$key])) {
					$headers[$key] = $value;
				} elseif (is_array($headers[$key])) {
					$headers[$key][] = $value;
				} else {
$headers[$key] = [$headers[$key], $value];
				}
			}
		} while ($line = strtok("\n"));
		
		$this->headers = (object) $headers;
		$this->response = strtok('');
	}

	public function get_response_format() {
		if (!$this->response) {
			throw new PulpoRestClientException(
				'A response must exist before it can be decoded.');
		}
		
		if (!empty($this->options['format'])) {
			return $this->options['format'];
		}

		if (!empty($this->headers->content_type)) {
			if (preg_match($this->options['format_regex'], $this->headers->content_type, $matches)) {
				return $matches[2];
			}
		}

		throw new PulpoRestClientException(
			'Response format could not be determined.');
	}
	
	public function decode_response() {
		if (empty($this->decoded_response)) {
			$format = $this->get_response_format();
			if (!array_key_exists($format, $this->options['decoders'])) {
				throw new PulpoRestClientException("'${format}' is not a supported " .
					'format, register a decoder to handle this response.');
			}
			
			$this->decoded_response = call_user_func(
				$this->options['decoders'][$format], $this->response);
		}
		
		return $this->decoded_response;
	}
}




